
;;      HangMan.cl - From the Forthcoming Volume:
;;
;;      The Natural Language Computing Workbook
;;                        By 
;;      Ray C. Dougherty Ph.D. and Marc Schwarz M.S.
;;
;;       Copyright 1996 Marc Schwarz and Ray Dougherty
;;                   All Rights Reserved.
;;      May be reprinted for non-commerical distribution
;;      providing that the code remain unaltered and that
;;      this copyright notice is included with the source
;;      code. 
;;
;; HangMan.cl
;; Version 11
;; Common LISP Implementation of the HangMan Game
;; Includes "pretty good" pseudo random number generating mechanism


(setq alphabet '(a b c d e f g h i j k l m n o p q r s t u v w x y z))

(setq primes '(13 17 19 23 29 31 37 41 43 47 53))

(setq const (length primes))

(setq word-database '(help three battle cello great spies blazing psycho freedom 
birds family knew much days came from cold high silent plot anxiety foxes wings
enigma enemy ceiling compute turtle tomato purple angeles window four submarine
saddles movie desert deadly britain midway escape express eyes night morning knot 
seven forget expand secret wanted binding slinky regret youthful regards extend
lotus conjunction armadillo cactus koala orchid wednesday society eucalyptus bent 
elephant drive basketball valley hockey forest political praire toast large govern
eagle freeway sharks pandas corporate elevator pirana trains plague rhino bridge
complicate raiders turnpike cholera newspaper calcium exponent gerund assertive 
pseudo sideways golden horse envelope peppermint orangutan platitude cider moose
orange carriage ostrich pardon cyclone mouse egress solitude spear lemon current 
rodeo apple aardvark saffron easy correct silicon graduate stupid person marsupial 
happened yellow hard ticket frenzy chicken bridge self shoes cricket spring below))

(setq maxval (length word-database))

(setq words-remain word-database)
(setq words-used '()) ;;used in debugging

;;; Begin Pseudo Random Number Generator Mechanism

;;; used to test number generating code
;;; (setq words-remain '(help three battle))
;;; (hangman)

(setq glorand 1)
(setq initc 0)

(setq seed 1)

;; Design notion: assume user will ALWAYS type in the same
;; number when instructed "please input a number from 1 to N"
(defun GetWord()
   (cond ((and (equal initc 0) (equal seed 1))
              (princ '|Please type in the year your were born|) 
	      (terpri) (terpri)
              (setq initc (+ 1 initc))
	      (GoGetWord1 initc seed))
         (t (setq initc (+ 1 initc))
            (setq seed (+ seed (* 11 
                                 (mod (+ 1 (* glorand (random 100))) 
                                      (+ 2 (round seed 2)))) ))
            (RandWord seed words-remain (length words-remain) initc))))      


(defun GoGetWord1(inic sed)
  (let ((Guess1 (read)))

 (cond ((or (< Guess1 1850) (> Guess1 6000)) (setq initc (- inic 1)) 
                                          (GetWord))
        (t
           (setq glorand Guess1) 
           (setq seed (+ sed (* (+ 1 (mod Guess1 7)) 
                                (mod (+ 1 (* Guess1 (random 100))) 
                                     (+ 2 (round sed 2)))) ))
           (RandWord sed words-remain (length words-remain) inic)))))


(defun RandWord(sed db lendb inic)
   (let* 
        ((loc  (Randomizer sed inic lendb)))

  (cond ((> sed 10000001) (setq seed (* (* inic 7) 
                                       (mod (+ 1 (round sed 2)) 10000))) 
                          (usedtest loc words-remain))
        ((and (equal inic 1) (> loc 65) (< loc 85)) (futest loc words-remain))
        (t (usedtest loc words-remain)))))

(defun Randomizer(sd inic lendb)
     (mod 
	(* (mod (+ inic sd) 
        	(+ maxval 
          	   (nth (mod (* 30 inic) const) primes)))
   	   (* (nth (mod inic const) primes) 
              (nth (mod (* 2 inic) const) primes)))
        lendb))

;; in the first run-through, the loc tends to be centralized
;; so force a wider range for loc
(defun futest(lc wrmn)
  (let* ((boost (* lc (* glorand glorand)))
         (newlc (+ 1 (mod  boost (length wrmn)))))
    (setq seed boost)
    (usedtest newlc wrmn)))

(defun usedtest(locr wordrem)
   (let* ((newword (nth locr wordrem)))

   (cond ((equal (length wordrem) 1)
            (setq words-remain (remove (car wordrem) word-database))
            (setq words-used (append '() (list (car wordrem))))
            (car wordrem))  ;;; note: (car wordrem) = newword
          (t  (setq words-remain (remove newword wordrem))
	      (setq words-used (append words-used (list newword)))
              newword))))


;;; Begin Program Proper


(defun HangMan()
   (let*
	((Word	(explode (GetWord)))
         (Found  '())     
	 (Used  '())
	 (Remaining alphabet))

	;;(terpri) (print Word)        ;; used in debugging
        ;;(print (list seed)) (terpri) ;; used in debugging

   	(princ '|Let's Play HangMan!|) (terpri) (terpri)
        (print (FoundSoFar Word Found '()))
        (terpri)
        (DrawGallows)
  	(DrawNoose 0 0)
        (terpri)
        (princ '|Letters Used: |)
  	(princ Used) (terpri)
        (princ '|Letters Unused: |)
  	(princ Remaining) (terpri)
        (terpri)
        (princ '|To exit the game at any time type "1" as your guess|)
        (terpri)
  	(princ '|PLEASE GUESS A LETTER>>|)

   	(PlayHangMan Word Found Used Remaining 0 0)
   	(PlayAgain)))


(defun PlayHangMan(Wrd Fnd Usd Rmn NumMissed NumWGuess)
  (let ((Guess (read)))

     (cond  ((equal GUESS '1) nil)  ;;;mass exit
            ((eq GUESS '!) (GuessWord Wrd Fnd Usd Rmn NumMissed NumWGuess))
            ((not (member GUESS alphabet))
              (princ '|PLEASE GUESS A **LETTER** THIS TIME>>|) 
              (PlayHangMan Wrd Fnd Usd Rmn NumMissed NumWGuess)) 
            ((member GUESS Usd)
              (princ '|PLEASE GUESS A LETTER THAT HAS NOT ALREADY BEEN CHOSEN>>|) 
              (PlayHangMan Wrd Fnd Usd Rmn NumMissed NumWGuess))    
           ((member GUESS Wrd) 
	      (ResolveG Wrd 
                        (append Fnd (list GUESS))
                        (append Usd (list GUESS))
                        (remove GUESS Rmn)
                        NumMissed
			NumWGuess))
           (t (ResolveG Wrd 
                        Fnd 
                        (append Usd (list GUESS))
                        (remove GUESS Rmn) 
                        (+ NumMissed 1)
			NumWGuess)))))


(defun ResolveG (Wrd NFnd NUsd NRmn NumMiss NumWGuess)
  (let* ((FSF (FoundSoFar Wrd NFnd '())))

    (terpri)
    (print FSF)
    (terpri)
    (DrawGallows)
    (DrawNoose 0 NumMiss)
    (terpri)
    (cond ((< NumMiss 7)
    	   (princ '|Letters Used: |)
    	   (princ NUsd) (terpri)
    	   (princ '|Letters Unused: |)
           (princ NRmn) (terpri)
           (terpri)
           (princ '|PLEASE GUESS A LETTER OR TYPE "!" TO GUESS THE WORD>>|)
           (PlayHangMan Wrd NFnd NUsd NRmn NumMiss NumWGuess))
         (t (princ '|The word you wanted was: |)
            (princ Wrd)) (terpri)))))


(defun FoundSoFar(Wrd Fnd FIN)
   (cond ((null Wrd) FIN)
         ((member (car Wrd) Fnd) 
	    (FoundSoFar (cdr Wrd) Fnd (append FIN (list (car Wrd)))))
         (t (FoundSoFar (cdr Wrd) Fnd (append FIN (list '*))))))


(defun GuessWord(Wrd Fnd Usd Rmn NumMissed NumWGuess)
  (cond ((equal NumWGuess 0)
             (terpri) 
             (princ '|You get three chances to guess the word|) 
             (ProcessGuess Wrd Fnd Usd Rmn NumMissed NumWGuess))
        (t (ProcessGuess Wrd Fnd Usd Rmn NumMissed NumWGuess))))


(defun ProcessGuess(Wrd Fnd Usd Rmn NumMissed NumWGuess)
   (terpri)
   (princ '|PLEASE TYPE IN WHAT YOU THINK THE WORD IS>>>|)
     (let ((WordGuess (read)))
        (GoGuessWord (explode WordGuess) 
                     Wrd 
                     Fnd 
                     Usd 
                     Rmn 
                     NumMissed
		     NumWGuess))))


(defun GoGuessWord(Gs Wrd Fnd Usd Rmn NumMissed NumWGuess)
  (let ((temp (- 3 (+ NumWGuess 1)))
        (FSF (FoundSoFar Wrd Gs '())))

   (cond ((equal Wrd Gs)
            (terpri)
            (print FSF)
            (terpri)
            (DrawGallows)
            (terpri)
            (princ '|   YOU WIN |) (terpri)
    	    (terpri) 
            (princ '|Congratulations!!! You correctly guessed the word|) 
            (terpri))
         ((equal NumWGuess 2)
            (princ '|NO... I'm sorry, your guess was incorrect|) (terpri)
            (princ '|That was also your third and final guess|) (terpri)
            (terpri)
            (princ '|The word you wanted was: |)
            (princ Wrd) (terpri))
         ((equal temp 1)  ;;added to correct grammar '1 chance'
            (princ '|NO... I'm sorry, your guess was incorrect|) (terpri)
            (princ '|You have |)
            (princ temp)
            (princ '| more chance to guess the word|) (terpri) (terpri)
            (princ '|PLEASE GUESS A LETTER OR TYPE "!" TO GUESS THE WORD>>|)
            (PlayHangMan Wrd Fnd Usd Rmn NumMissed (+ 1 NumWGuess)))
         (t (princ '|NO... I'm sorry, your guess was incorrect|) (terpri)
            (princ '|You have |)
            (princ temp)
            (princ '| more chances to guess the word|) (terpri) (terpri)
            (princ '|PLEASE GUESS A LETTER OR TYPE "!" TO GUESS THE WORD>>|)
            (PlayHangMan Wrd Fnd Usd Rmn NumMissed (+ 1 NumWGuess))))))
                  

(defun PlayAgain()
        (terpri)
        (princ '|Would you like to play again? (please input y/n) >|)
        (let ((reply (read)))

        (cond ((or (eq reply 'YES) 
                   (eq reply 'yes) 
                   (eq reply 'Y) 
                   (eq reply 'y)) (HangMan))
              (t (princ '|Thank you for playing HangMan.  Goodbye.|)
                 nil))))


(defun DrawNoose(Counter NumM)
   (cond  ((equal Counter 8) (YOU-LOSE))
          ((> Counter NumM) nil)
	  ((eq NumM 0) (terpri) (princ '|    CLEAR|) (terpri))
          ((eq Counter 1) (princ '|     0|) (terpri)
                                       (DrawNoose (+ Counter 1) NumM))
	  ((and (eq Counter 2) (eq NumM 2))
                          (princ '|     ()|) (terpri)
                                       (DrawNoose (+ Counter 1) NumM))
	  ((and (eq Counter 3) (eq NumM 3))
                          (princ '|   *-()|) (terpri)
                                       (DrawNoose (+ Counter 1) NumM))
	  ((eq Counter 4) (princ '|   *-()-*|) (terpri)
                                       (DrawNoose (+ Counter 1) NumM))
	  ((and (eq Counter 5) (eq NumM 5))
                          (princ '|    _/|) (terpri)
                                       (DrawNoose (+ Counter 1) NumM))
	  ((eq Counter 6) (princ '|    _/L|) (terpri)
                                       (DrawNoose (+ Counter 1) NumM))
	  ((eq Counter 7) (princ '|    RIP|) (terpri) (terpri)
                                       (DrawNoose (+ Counter 1) NumM))
	  (t (DrawNoose (+ Counter 1) NumM))))


(defun DrawGallows()
   (princ '|______+|) (terpri)
   (princ '|      :|) (terpri)
   (princ '|      :|) (terpri))


(defun YOU-LOSE()
   (princ '|Look's like your're at the end of your rope!|) (terpri) (terpri)) 

(defun explode(word)
   (let*
       ((lenword (length (string word))))

    (cond ((null word) nil)
          (t (goexplode word 1 lenword '())))))

(defun goexplode(word cur last fword)
   (cond ((equal cur last)
	   (append fword
                   (list (prog1 
                          (intern (string (char (string word) (1- cur)))))) ))

         (t  (goexplode word 
                        (+ cur 1) 
                        last
                        (append fword
                                (list (prog1 
                                        (intern 
                                        (string 
                                        (char (string word) (1- cur))))))) ))))




