
;;      HiLow.cl - From the Forthcoming Volume:
;;
;;      The Natural Language Computing Workbook
;;                        By 
;;      Ray C. Dougherty Ph.D. and Marc Schwarz M.S.
;;
;;       Copyright 1996 Marc Schwarz and Ray Dougherty
;;                   All Rights Reserved.
;;      May be reprinted for non-commerical distribution
;;      providing that the code remain unaltered and that
;;      this copyright notice is included with the source
;;      code. 
;;
;; HiLow.cl
;; Version 1a
;; Common LISP Implementation of the High-Low Number Guessing Game


;;;	High/Low Guessing Game
;;;
;;;	Used To Teach Notion of Procedures

;;;	METHOD:
;;;		I.	Randomly Pick Number from 1 to 100
;;;		II.	Prompt User for a GUESS (read inputted guess)
;;;		III.	Compare GUESS with Actual Number
;;;			    If GUESS > Actual, then print HIGH and goto II.
;;;			    If GUESS < Actual, then print LOW and goto II.
;;;			    If Guess = Actual, then print CORRECT and goto IV.
;;;		IV.	Ask User if he/she wants to play again
;;;			If YES, then goto I; else retrun NIL

;;; Preliminary "Prototype" Version

(defun HiLowMKI()
   (let*
	((Number	(car (PickNumber))))
	
   	;; (print Number) 
	;; printing the number simplifies the debugging process
	;; comment out the print statement once the program is debugged
   (terpri)
   (princ '|Let's Play High-Low!|) (terpri)
   (princ '|The System is Picking a Number Between 1 and 100.|) (terpri)
   (princ '|Please Guess the Number, or type 'q' to quit current game at anytime.>|)
   (PlayGameMKI Number 1) 
   (PlayAgainMKI)))

(defun PickNumber()
   (let*
	((N	(Random 100)))
   
	(cond ((eq N 0) (PickNumber))
	      (t (list N)))))

(defun PlayGameMKI(Num attempts)
	
	(let ((Guess (read)))

	(cond 

	      ((or (eq Guess 'q) (eq Guess 'Q)) 
			(princ '|Quiting current game in progress.|) (terpri))

	      ((> Guess Num) 	(princ '|Your Guess Was HIGH.  Please guess again.>|) 
				(PlayGameMKI Num (+ 1 attempts)))

	      ((< Guess Num) 	(princ '|Your Guess Was LOW.  Please guess again.>|) 
				(PlayGameMKI Num (+ 1 attempts)))

	      ((= Guess Num) 	(princ '|CORRECT GUESS --- Congratulations!|) (terpri)
		(princ '|You guessed the number in |)
                (princ attempts)
                (princ '| attempts.|) (terpri) ))))
				
(defun PlayAgainMKI()
	(princ '|Would you like to play again? (please input y/n) >|)
	(let ((reply (read)))

	(cond ((or (eq reply 'YES) 
		   (eq reply 'yes) 
		   (eq reply 'Y) 
		   (eq reply 'y)) (HiLow))
	      (t (princ '|Thank you for playing HighLow.  Goodbye.|)
                 nil))))


;;;;;;;;;;
;;;;;;;;;;

;;; 	HiLow
;;;
;;; Slightly modified version of above game.
;;; Allows player to quit and exit game in progress
;;; without having to answer play again


(defun HiLow()
   (let*
	((Number	(car (PickNumber))))
	
   ;; (print Number) ;; Used in debugging

   (terpri)
   (princ '|Let's Play High-Low!|) (terpri)
   (princ '|The System is Picking a Number Between 1 and 100.|) (terpri)
   (princ '|Please Guess the Number, or type 'q' to quit current game at anytime.>|)
   (PlayGame Number 1) ))


(defun PlayGame(Num attempts)
	
	(let ((Guess (read)))

	(cond 

	      ((or (eq Guess 'q) (eq Guess 'Q)) 
			;;(princ '|Quiting current game in progress.|) (terpri)
			(princ '|Thank you for playing HighLow.  Goodbye.|) nil)

	      ((> Guess Num) 	(princ '|Your Guess Was HIGH.  Please guess again.>|) 
				(PlayGame Num (+ 1 attempts)))

	      ((< Guess Num) 	(princ '|Your Guess Was LOW.  Please guess again.>|) 
				(PlayGame Num (+ 1 attempts)))

	      ((= Guess Num) 	(princ '|CORRECT GUESS --- Congratulations!|) (terpri)
		(princ '|You guessed the number in |)
                (princ attempts)
                (princ '| attempts.|) (terpri) 

		(PlayAgain) ))))

(defun PlayAgain()
	(princ '|Would you like to play again? (please input y/n) >|)
	(let ((reply (read)))

	(cond ((or (eq reply 'YES) 
		   (eq reply 'yes) 
		   (eq reply 'Y) 
		   (eq reply 'y)) (HiLow))
	      (t (princ '|Thank you for playing HighLow.  Goodbye.|)
                 nil))))
				
