
;;      Stats.cl - From the Forthcoming Volume:
;;
;;      The Natural Language Computing Workbook
;;                        By 
;;      Ray C. Dougherty Ph.D. and Marc Schwarz M.S.
;;
;;        Copyright 1997 Marc Schwarz and Ray Dougherty
;;                   All Rights Reserved.
;;      May be reprinted for non-commerical distribution
;;      providing that the code remain unaltered and that
;;      this copyright notice is included with the source
;;      code. 
;;
;; Stats.cl
;; Version 5
;; A Program For Generating Summary Statistics

(defun stats(inlist)
   (let* ((numlen (GetNumLen inlist 0 '()))
          (sortback (sort (cadr numlen) '<)))
    (cond ((null inlist) nil)
          (t (genstats (car numlen) sortback)))))


(defun GetNumLen(inlist count finlist)
   (let ((curhead (car inlist)))

    (cond ((null inlist) (cons count (list finlist)))
          (t (GetNumLen      (cdr inlist)
                            (+ count 1) 
                            (append finlist (list curhead)))))))


(defun genstats(count inlist)
  (let*
     ((hi (car (last inlist)))
      (low (car inlist))
      (range (cons low (list hi)))
      (mean (getmean count inlist))
      (median (getmedian count inlist))
      (mode (getmode (reverse inlist)))
      (value (car mode))
      (freq  (cadr mode))
      (standev (getstandev inlist count mean))
      (numofitems count)) 

(terpri)(terpri)
(princ '|Summary Statistics For Input List|) (terpri)
(princ '|---------------------------------|) (terpri)
(princ '|HIGH:                |) (princ hi) (terpri)
(princ '|LOW:                 |) (princ low) (terpri)
(princ '|RANGE:               |) (princ low) (princ '| - |) (princ hi) (terpri)
(princ '|MEAN:                |) (princ mean) (terpri)
(princ '|MEDIAN:              |) (princ median) (terpri) 
(princ '|MODE (Highest Value) |) (princ value) (terpri)
(princ '|MODE (With Highest Frequency): |) (princ freq) (terpri)  
(princ '|STANDARD DEVIATION:  |) (princ standev) (terpri)
(princ '|NUMBER OF ITEMS:     |) (princ count) (terpri)(terpri)(terpri)))


(defun GetMedian(count inlist)
   (cond ((null inlist) nil)
         (t (nth (floor (/ (- count 1) 2)) inlist))))

(defun getmean(count inlist)
  (if (null inlist) nil
   (let ((sumback (GetSum inlist 0.0)))
         (/ sumback (float count)))))

(defun GetSum(inlist tot)
   (cond ((null inlist) tot)
         (t (GetSum (cdr inlist) (+ tot (car inlist))))))

(defun GetMode(inlist)
  (if (null inlist) nil
   (let* ((modeset (GetCount (car inlist) (cdr inlist) 1 '()))
          (firstmode (car modeset))
          (remlist   (cadr modeset)))
    (cond ((null remlist) firstmode)
          (t (GoGetMode firstmode remlist))))))


(defun GoGetMode(fmode rlist)
  (if (null rlist) fmode

  (let* ((newmodeset (GetCount (car rlist) (cdr rlist) 1 '()))
         (newmode (car newmodeset))
         (newremlist (cadr newmodeset)))

   (cond ((and (< (cadr fmode) (cadr newmode)) (null newremlist)) newmode)
         ((and (> (cadr fmode) (cadr newmode)) (null newremlist)) fmode)
         ((< (cadr fmode) (cadr newmode)) (GoGetMode newmode newremlist))
         ((>= (cadr fmode) (cadr newmode)) (GoGetMode fmode newremlist))))))


(defun GetCount(head remlist headcount finlist)
   (cond ((null remlist)
              (append (list (cons head (list headcount))) (list finlist)))
         ((equal head (car remlist))
              (GetCount head (cdr remlist) (+ headcount 1) finlist))
         (t (append  (list (cons head (list headcount)))
                     (append finlist (list remlist))))))


(defun GetStanDev(inlist count mean)
   (if (null inlist) nil

    (let* 
         ((SumOfSquares (GetSumSquares inlist 0))
          (AVGSumOfSquares (/ SumOfSquares (float count)))
          (DiffOfAVGS (- AVGSumOfSquares (float (* mean mean)))))

      (sqrt DiffOfAVGS))))

(defun GetSumSquares(inl tot)
   (cond ((null inl) tot)
         (t (GetSumSquares (cdr inl) (+ tot (* (car inl) (car inl)))))))



